/* inlist.c */
/* $Id: inlist.c,v 1.3 1995/04/03 06:49:13 jt Exp jt $ */
/* tools for dealing with this kind of list:
 *    3-5,8,10-12     ==> 3,4,5,8,10,11,12
 *    -6,10-          ==> 1,2,3,4,5,6, 10,11,12,...EOF
 * Note that this is a 1-based list, you cant have 0-5, -3--1, etc.
 */
#include <stdio.h>
#include <ctype.h>

#ifndef DEBUG
#  define DEBUG 0
#endif

static char *
nvalue(char *cur, int *n)
{
    if (!isdigit(*cur)) {
	*n = -1;
	return cur;
    }
    *n = 0;
    while ( isdigit(*cur) ) {
	*n = 10*(*n) + (*cur - '0');
	++cur;
    }
    return cur;
}
static char *
nvalues(char *cur, int *nstart, int *nend)
{
    *nstart = *nend = 0;
    /* skip past leading commas & leading whitespace */
    while (*cur == ',' || isspace(*cur))
	++cur;
    if (DEBUG) fprintf(stderr,"<%c>",*cur);
    if (*cur == '-') {
	*nstart = -1;
	cur = nvalue(++cur,nend);
    } else if (isdigit(*cur)) {
	cur = nvalue(cur,nstart);
	*nend=*nstart;
	if (*cur == '-') {
	    cur = nvalue(++cur,nend);
	}
    } else {
	fprintf(stderr,"Invalid string \"%s\"\n",cur);
	++cur;
    }
    return cur;
}

/* hvalue: gets hex value from string pointed to by 'cur',
 * and returns pointer to next character past the number
 */
static char *
hvalue(char *cur, int *n)
{
    /* skip past leading 0x */
    if (*cur == '0' && *(cur+1) == 'x')
        cur += 2;
    
    if (!isxdigit(*cur)) {
	*n = -1;
	return cur;
    }
    *n = 0;
    while ( isxdigit(*cur) ) {
        if (isdigit(*cur)) {
            *n = 16*(*n) + (*cur - '0');
        } else {
            *n = 16*(*n) + (tolower(*cur) - 'a' + 10);
        }
	++cur;
    }
    return cur;
}

static char *
hvalues(char *cur, int *nstart, int *nend)
{
    *nstart = *nend = 0;
    if (DEBUG) fprintf(stderr,"<%c>",*cur);
    if (*cur == '-') {
	*nstart = -1;
	cur = hvalue(++cur,nend);
    } else if (*cur == ',') {
	*nstart = *nend = 0;
	++cur;
    } else if (isxdigit(*cur)) {
	cur = hvalue(cur,nstart);
	*nend=*nstart;
	if (*cur == '-') {
	    cur = hvalue(++cur,nend);
	}
    } else {
	fprintf(stderr,"Invalid string \"%s\"\n",cur);
    }
    return cur;
}
int
maxinlist(char *list)
{
    int start,end;
    int maxend=0;
    char *cur;
    cur = list;
    while (*cur != '\0') {
	cur = nvalues(cur,&start,&end);
	if (end == -1)
	    return -1;
	if (end > maxend)
	    maxend = end;
    }
    return maxend;
}
	
int
inlist(char *list, int i)
{
    int start,end;
    char *cur;
    cur = list;
    while (*cur != '\0') {
	cur = nvalues(cur,&start,&end);
	if (DEBUG) fprintf(stderr,"[%d:%d-%d]",i,start,end);
	if (end == -1)
	    if (i >= start) return 1;
	if (start == -1)
	    if (i <= end) return 1;
	if (i <= end && i >= start) 
	    return 1;
    }
    return 0;
}
int
inlist2array(char *list, int max, int *array)
{
    int start,end;
    int i,j=0;
    char *cur;
    cur = list;
    while (*cur != '\0' && j<max) {
	cur = nvalues(cur,&start,&end);
	if (DEBUG) fprintf(stderr,"[%d-%d]",start,end);
	if (end == -1)
	    end = max;
	if (start == -1)
	    start = 0;
	for (i=start; i<=end && j<max; ++i) {
	    if (DEBUG) fprintf(stderr,"array[%d]=%d\n",j,i);
	    array[j++] = i;
	}
    }
    return j;
}
int
hinlist(char *list,int i)
{
    int start,end;
    char *cur;
    cur = list;
    while (*cur != '\0') {
	cur = hvalues(cur,&start,&end);
	if (DEBUG) fprintf(stderr,"[%d:%d-%d]",i,start,end);
	if (end == -1)
	    if (i >= start) return 1;
	if (start == -1)
	    if (i <= end) return 1;
	if (i <= end && i >= start) 
	    return 1;
    }
    return 0;
}

#ifdef MAIN
#define MAX 25
printfulllist(char *list)
{
    int i;
    printf("\n");
    for(i=1; i<MAX; ++i) {
	if (inlist(list,i)) {
	    printf("%d,",i);
	}
    }
    printf("\n");
}

main(int argc, char **argv) 
{
    char *list;

    if (argc != 2) {
	fprintf(stderr,"Usage: %s list\n",argv[0]);
	fprintf(stderr,"   where list is something like: 3-5,8,13-21\n");
	exit (-1);
    }
  
    list = argv[1];

    printfulllist(list);
    return 0;
}
#endif
