/* $Id: svd.c,v 1.1 1995/04/03 05:50:55 jt Exp jt $ */

#include <stdio.h> /* for debugging */
#include <math.h>  /* for sqrt */
#include "nr.h"
#include "nrutil.h"
#ifndef EPSILON
#define EPSILON 0.000001
#endif

extern int verbose;

svd_solve(float **x, float *y, float *a,
	  int n, int m, float **u, float *w, float **v)
{
    int i,j;
    float wmax;

    for (i=1; i<=n; ++i)
	for (j=1; j<=m; ++j)
	    u[i][j] = x[i][j];
    
    svdcmp(u,n,m,w,v);

    wmax = 0;
    for (j=1; j<=m; ++j) 
	if (w[j] > wmax)
	    wmax = w[j];

    if (verbose)
	fprintf(stderr,"Singular values:\n");
    for (j=1; j<=m; ++j) {
        if (verbose)
            fprintf(stderr,"w[%d] = %g",j,w[j]);
	if (w[j] < EPSILON*wmax) {
	    if (verbose)
                fprintf(stderr," --> 0");
	    w[j]=0;
	}
        if (verbose)
            fprintf(stderr,"\n");
    }

    svbksb(u,w,v,n,m,y,a);
}

int
fit_report(float *w, float **v, int m)
{
    int j,k;
    float sum; 
    fprintf(stderr,"Error Bars:\n");
    for (j=1; j<=m; ++j) {
	sum = 0;
	for (k=1; k<=m; ++k) {
	    if (w[k] != 0) {
		sum += v[j][k]*v[j][k]/(w[k]*w[k]);
	    }
	}
	fprintf(stderr, "err_a[%d] = %g\n",j,sqrt(sum));
    }

    if (verbose > 1) {
	fprintf(stderr,"Columns are Principal Components:\n");
	fprintf(stderr,"< ");
	for (k=1; k<=m; ++k) {
	    fprintf(stderr,"%8g%s",w[k],(k==m ? " >\n" : " "));
	}
	for (j=1; j<=m; ++j) {
	    fprintf(stderr,"[ ");
	    for (k=1; k<=m; ++k) {
		fprintf(stderr,"%8.5lf%s",v[j][k],(k==m ? " ]\n" : " "));
	    }
	}
    }
}
