package ConfidenceInterval;
my $verbose=1;    
sub new { 
    my $class = shift;
    $class = ref($class) || $class; # see Camel Book, p295
    my $self = {};                  # All Std Objects are hashes
    bless $self,$class;
    $self->init(@_);
    return $self;
}
sub init {
    my $self = shift;
    my $name = shift || "";
    $self->{name}  = $name;
    $self->{data}  = [];
    $self->{count} = 0;
}
sub addData {
    my $self = shift;
    my $data = shift || 0;
    push @{$self->{data}},$data;
    $self->{count} += 1;
    return $self;
}
sub print {
    my $self = shift;
    print STDERR 
	$self->{name},
	", data=",${$self->{data}}[0]," ...",
	", cnt=",$self->{count},
	"\n";
}
sub mean {
    my $self = shift;
    my $cnt = $self->{count};
    my $sum = 0;
    for ($i=0; $i<$cnt; ++$i) {
	$sum += $self->{data}->[$i];
    }
    return ($sum/$cnt);
}
sub median {
    my $self = shift;
    my $cnt = $self->{count};
    my @values = @{$self->{data}};
    my @sorted = sort @values;
    my $halfcnt = int($cnt/2);
    return $sorted[$halfcnt];
}
sub variance {
    my $self = shift;
    my $cnt = $self->{count};
    my $ave = $self->mean;
    my $sum = 0;
    for ($i=0; $i<$cnt; ++$i) {
	my $d = $self->{data}->[$i];
	$d -= $ave;
	$sum += $d*$d;
    }
    return ($sum/$cnt);
}
sub stddev {
    my $self = shift;
    return sqrt($self->variance);
}
sub interval {
    my $self = shift;
    my $confidence = shift || 0.95;
    warn "Invalid confidence ($confidence)"
	if ($confidence < 0 || $confidence > 1.0);
    my $cnt = $self->{count};
    my $lo = (0.5-$confidence/2)*($cnt+1) - 1;
    my $hi = (0.5+$confidence/2)*($cnt+1) - 1;
    my @tmp = sort @{$self->{data}};
    my $dlo = $lo - int($lo);  $lo = int($lo);
    my $dhi = $hi - int($hi);  $hi = int($hi);
    if ($dlo < 0) { $tlo = $tmp[0]; }
    else          { $tlo = $tmp[$lo]*(1-$dlo) + $tmp[$lo+1]*$dlo; }
    if ($hi == $cnt - 1) { 
	$thi = $tmp[$hi];
    } else {
	$thi = $tmp[$hi]*(1-$dhi) + $tmp[$hi+1]*$dhi;
    }
    if ($verbose) {
	print STDERR 
	    "lo=$lo, dlo=$dlo, hi=$hi, dhi=$dhi, cnt=$cnt, [$tlo,$thi]\n";
    }
    return ($tlo,$thi);
}
    
1;
__END__

@Usage = <<EU;
Computes the mean and variance of data as it arrives.
Data is added one at a time using 'addData' method; 
then mean and variance are provided when asked, using
'mean' and 'variance' methods.
EU
