#!/bin/sh
## run-max-like
## Usage: run-max-like <TreeFile> <NodeNumber> <FilePrefix> <BootStraps>

if [ $# -lt 4 ]; then
    echo "$0 <TreeFile> <NodeNumber> <FilePrefix> <nBoot> <noZ59> <lin> <norm>"
    echo "<nBoot> is number of bootstraps"
    echo "<noZ59> is either 1 or 0, specifies whether to *exclude* Z59 node"
    echo "<lin>   is either 1 or 0, specifies use of linfit in bootstraps"
    echo "<norm>  is a normalization factor, usually near 1"
    exit
fi   

treefile=$1
node=$2
file=$3
nboot=$4

Z=$5
linfit=$6

norm=$7
if [ -z "$norm" ]; then norm=1; fi 

#### The following have to be changed by hand
over=1
noAncestors=1

echo $0
echo treefile=$treefile
echo node=$node
echo file=$file
echo nboot=$nboot
echo Z=$Z
echo linfit=$6
echo norm=$7

#### over=1 is default, if it is a larger number (always integer), then
#### the bootstraps are made with much larger data sets...this means they
#### provide invalid error bars, but test the question of whether the
#### max-likelihood (or linear fit) consistently reproduce the model
#### parameters in the limit of large data sets


### This only works on Phylip TreeFile's
bases=`grep Sites $treefile | awk '{print $3}'`
echo Infer Bases=$bases from TreeFile=$treefile
 
branchlength -noAncestors=$noAncestors -noZ59=$Z -B=$bases -Yo=1990 -norm=$norm \
             $node $treefile > $file.in
branchlength -noAncestors=$noAncestors -noZ59=$Z -norm=$norm -xyfile=$file.xy \
             $node $treefile >/dev/null

## Linear Fits
## First, plot time vs distance (backwards)

b=`linfit -m1 -Y0 -X1 $file.in | head -1`
a=`linfit -m1 -Y0 -X1 $file.in | tail -1`

echo Linear Fit Parameters 1/a=$a b=$b
yr=`echo "scale=3; 1990+($b)" | bc`
dx=`echo "scale=3; ((10-($b))/$a)/$bases" | bc`
echo "#k"        > $file-linfit-back.xy
echo "$yr  0"   >> $file-linfit-back.xy
echo "2000 $dx" >> $file-linfit-back.xy

## Next, plot distance vs time (normal)

b=`linfit -m1 -Y1 -X0 $file.in | head -1`
a=`linfit -m1 -Y1 -X0 $file.in | tail -1`

echo Linear Fit Parameters a=$a b=$b
yr=`echo "scale=3; 1990-$b/$a" | bc`
dx=`echo "scale=3; ($b + 10*$a)/$bases" | bc`
echo "#k"        > $file-linfit.xy
echo "$yr  0"   >> $file-linfit.xy
echo "2000 $dx" >> $file-linfit.xy

echo a=$a b=$b
max-like -a$a -b$b -t3.0 -r2.5 -wt < $file.in

echo Running Maximum Likelihood On Data
run-max-like -xyfile=$file-bestfit.xy -a=$a -b=$b -B=$bases $file.in |\
    tee $file-tmp.out

####################################################################

a=`tail -1 $file-tmp.out | awk '{print $6}'`
b=`tail -1 $file-tmp.out | awk '{print $8}'`
t=`tail -1 $file-tmp.out | awk '{print $10}'`
l=`tail -1 $file-tmp.out | awk '{print $12}'`

echo Maximum Likelihood Parameters a=$a b=$b t=$t, like=$l

yr=`echo "scale=3; 1990-$b/$a" | bc`
dx=`echo "scale=3; ($b + 10*$a)/$bases" | bc`

echo Maximum Likelihood Parameters yr=$yr dx=$dx

echo "#k"        > $file-maxlikefit.xy
echo "$yr  0"   >> $file-maxlikefit.xy
echo "2000 $dx" >> $file-maxlikefit.xy

pois-error -a=$a -b=$b -B=$bases > $file-pois-errbar.xy

if [ $nboot -eq 0 ]; then
    exit
fi

## Now you should use the values of a,b,tau as starting points
## in the bootstrap calculations

save=0
bmlopt="-a=$a -b=$b -t=$t -nboot=$nboot -linfit=$linfit -over=$over -save=$save";

## dont bother with Monte-Carlo
#echo Running $nboot Monte-Carlo simulations
#boot-max-like $bmlopt $file.in > $file-b$nboot.out 2> $file-b$nboot.err

## Get confidence intervals; and slopes as xyplots

#meanvar -xy $file-b$nboot.err       > $file-b$nboot-ci.xy
#errtoxy -B=$bases $file-b$nboot.err > $file-b$nboot.xy

# run with Efron-style bootstrap
echo Running $nboot Efron-style bootstraps
echo bmlopt = $bmlopt
boot-max-like -efron $bmlopt $file.in > $file-bs$nboot.out 2> $file-bs$nboot.err

/bin/rm $file-bs$nboot.out
meanvar -xy $file-bs$nboot.err       > $file-bs$nboot-ci.xy
errtoxy -B=$bases $file-bs$nboot.err > $file-bs$nboot.xy


