#!/usr/bin/perl -s
##
($me = $0) =~ s,.*/,,;
my @Usage = 
    ("Usage: $me [-a=# -b=# -t=#] [-fixtau] [-linfit] file.in",
     "   Runs max likelihood estimator for a model with slope (a)",
     "   intercept (b), and delay (t), using as starting values",
     "   a,b,t as given on command line.",
     "   Input file has two columns; first is year, ",
     "   second is distance to node, in (integer) base-changes.",
     "   Writes to stdout enough information for a nice xyplot of",
     "   convergence.  Writes final answer to stderr.",
     );
sub usage {
    print join("\n",@Usage),"\n";
    exit 0;
}

$a ||= 6.93952;  $ra=3.0;
$b ||= 421.405;  $rb=50.0;
$t ||= 3.57597;  $rt=1.5;
$B ||= 2038;
$xyfile ||= "";
$fixtau ||= 0;
$linfit ||= 0;

$t=0 if ($t < 0);

$infile= shift @ARGV || "node51-8804-like.in";
&usage unless -f $infile;

$N = `wc -l < $infile`;
$N =~ s/\s*//g;

if ($linfit) {
    open(LINFIT,"linfit -m1 -Y0 -X1 $infile |");
    ($a,$b)=<LINFIT>;
    close LINFIT;
    print STDERR "run-max-like it= 0 a= $a  b= $b tau= 0 like= 0\n";
    exit;
}


$it=0;
$eps=0.5;
while ($it < 60) {

    $eps = ($eps < 0.0005) ? 0.0001 : $eps/5;
    $tol = $eps*$a;
    open(AFILE,"max-like -e$tol -N$N -a$a -b$b -t$t -wa -r$ra < $infile |") or die;
    $line = <AFILE>;
    @vals = split(/\s+/,$line);
    $da = $a - $vals[1];
    printf "%.5f %.5f %.5f %.5f da=%.5f\n",@vals[(1,2,3,5)],$da;
    $da = ($da>0 ? $da : -$da);
    $ra = ($ra/3 > 2*$da ? $ra/3 : 2*$da);
    $a = $vals[1];

    $tol = $eps*$b;
    open(BFILE,"max-like -e$tol -N$N -a$a -b$b -t$t -wb -r$rb < $infile |") or die;
    $line = <BFILE>;
    @vals = split(/\s+/,$line);
    $db = $b - $vals[2];
    printf "%.5f %.5f %.5f %.5f db=%.5f\n",@vals[(1,2,3,5)],$db;
    $db = ($db>0 ? $db : -$db);
    $rb = ($rb/3 > 2*$db ? $rb/3 : 2*$db);
    $b = $vals[2];

    $rt=0 if ($fixtau);
    open(TFILE,"max-like -e$eps -N$N -a$a -b$b -t$t -wt -r$rt < $infile |") or die;
    $line = <TFILE>;
    @vals = split(/\s+/,$line);
    $dt = $t - $vals[3];
    printf "%.5f %.5f %.5f %.5f dt=%.5f\n",@vals[(1,2,3,5)],$dt;
    $dt = ($dt>0 ? $dt : -$dt);
    $rt = ($rt/3 > 2*$dt ? $rt/3 : 2*$dt);
    $t = $vals[3];
    $rt = ($rt < $t ? $rt : 0.9*$t);

    ##printf "<%02d> rda=%.5f rdb=%.5f rdt=%.5f\n",$it,$da/$a,$db/$b,$dt/$t;
    last if ($it>6 && $eps < 0.0002 
	     && $ra/$a < 0.002 && $rb/$b < 0.002 && $rt/($t+1) < 0.002);
    ++$it;

}

print "#k run-max-like it= $it a= $a  b= $b tau= $t like= $vals[5]\n";
if ($xyfile) {
    open(BESTFIT,">$xyfile") || warn "Cant open xyfile=[$xyfile]";
    print BESTFIT "#k a= $a  b= $b tau= $t like= $vals[5]\n";
    print BESTFIT 1990-$b/$a," 0\n";
    print BESTFIT "2000. ",($b+10*$a)/$B,"\n";
}
print STDERR "run-max-like it= $it a= $a  b= $b tau= $t like= $vals[5]\n";

__END__

