/* max-like.c */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "like.h"
#include "linmin.h"
#include "dlnfact.h"

#include "opt.h"

#define BRENT_TOL 0.001
double STEPS_PER_TAU=50;
/* extern double qromb_eps; */
extern qromb_eps;

void rand_unitsphere(double *px, double *py, double *pz);
double brent(double ax, double bx, double cx, double (*f)(double), double tol,
            double *xmin);

int plotFlag=0;
int Npoints=2000;
double tol=BRENT_TOL;

void
run(double a, double b, double tau, char which, double range)
{     
    int i;
    double x;
    double mxlike;
    double d[3];
    Data_t *data;
    double l;
    
    data = (Data_t *)calloc(Npoints,sizeof(Data_t));
    for (i=0; i<Npoints; ++i) {
        if (2 != fscanf(stdin,"%lf %d\n",&(data[i].t),&(data[i].n)))
            Npoints=i;
    }
    fcn_init_data(Npoints,data);
    reset_count();

    d[0]=d[1]=d[2]=0;
    switch (which) {
    case 'a': d[0]=1; break;
    case 'b': d[1]=1; break;
    case 't': d[2]=1; break;
    case 'r':
    default:
        rand_unitsphere(&d[0],&d[1],&d[2]);
        break;
    }
        
    fcn_init_parm(a,b,tau,d[0],d[1],d[2]);
    reset_count();

    if (range > 0) {
        if (plotFlag) {
            for (x= -range; x<range; x+=range/30) {
                l = -1.0*fcn1d(x);
                fprintf(stdout,"%g %g\n",x,l);
            }
        }
        /* NR */
        mxlike = -1.0*brent(-range,0,range,fcn1d,tol,&x);
    }
    else {
        x = 0;
        mxlike = -1.0*fcn1d(0.0);
    }
    
    a   += x*d[0];
    b   += x*d[1];
    tau += x*d[2];

    if (plotFlag) {
        fprintf(stderr,"parms= %.13g %.13g %.13g mxlike= %.13g\n",
                a,b,tau,mxlike);
    }
    else {
        fprintf(stdout,"parms= %.13g %.13g %.13g mxlike= %.13g\n",
                a,b,tau,mxlike);
    }
    free((char *)data);
}

int
main(int argc, char **argv)
{
    double
        a=10.0,
        b=6.5,
        tau=1.0;
    char which='a';
    double range=1.0;
    optreg(&a,OPT_DOUBLE,'a',  "Slope of fit line     (initial guess)");
    optreg(&b,OPT_DOUBLE,'b',  "Intercept of fit line (initial guess)");
    optreg(&tau,OPT_DOUBLE,'t',"Time error bar: tau   (initial guess)");
    optreg(&tol,OPT_DOUBLE,'e',"Epsilon: precision of final answer");
    optreg(&which,OPT_CHAR,'w',"Which to minimize: -wa, -wb, or -wt");
    optreg(&range,OPT_DOUBLE,'r',"Range over which to attempt maximization");
    optreg(&plotFlag,OPT_FLAG,'p',"Plot likelihood over the range");
    optreg(&Npoints,OPT_INT,'N',"Number of points");
    opt(&argc,&argv);
    STEPS_PER_TAU = 10 + .05/tol;
    qromb_eps = tol/100;
    if (qromb_eps < 1.0e-5) qromb_eps=1.0e-5;
    if (a == 0) a=10.0;
    run(a,b,tau,which,range);
    return 0;
}
