/* mc.c */
/* Monte-Carlo generation of random data */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "opt.h"

double rand_unif(void);
double poidev(double);

int N=1;
int seed=17;
int verbose=0;
int tosszero=0;
double a=10.0,b=100.0,tau=1.0;
char *infile=NULL;

int run(void);
main(int argc, char **argv)
{
    optreg(&a,OPT_DOUBLE,'a',  "Slope of line");
    optreg(&b,OPT_DOUBLE,'b',  "Intercept of line");
    optreg(&tau,OPT_DOUBLE,'t',"Time error bar: tau ");
    optreg(&seed,OPT_INT,'s',"Random number seed");
    optreg(&infile,OPT_STRING,'I',"Input data file (only t-values used)");
    optreg(&N,OPT_INT,'O',"Oversampling: MC points per data point");
    optreg(&tosszero,OPT_FLAG,'z',"Avoid zeros");
    optreg(&verbose,OPT_FLAG,'v',"Verbose");
    optRun(run);
    opt(&argc,&argv);
    return run();
}

run()
{
    int i;
    double t,to,mu;
    int n;
    FILE *fp;

    /* Open input file */
    fp = fopen(infile,"r");
    if (fp == NULL) {
        fprintf(stderr,"Cannot open infile: [%s]\n",infile);
        return -1;
    }

    /* Seed rng */
    rand_seed(seed);

    /* Loop over data points in file */
    while( 1 == fscanf(fp,"%lf %*d",&to)) {
        /* Oversample: N points per data point in file */
        for (i=0; i<N; ++i) {
            do {
                t = to+(rand_unif()-0.5);      /* time of actual sampling */
                if (verbose) fprintf(stderr,"to=%g t=%g ",to,t);
                t += tau*log(rand_unif());     /* time of creation of this species
                                                * on average, tau in the past */
                mu = a*t+b;                    /* expected number of mutations */
            } while (tosszero && mu<0);
            n = poidev(mu);                /* actual number */
            if (verbose) fprintf(stderr,"t=%g mu=%g n=%d\n",t,mu,n);
            fprintf(stdout,"%g %d\n",to,n);
        }
    }
    
    return 0;
    
}
